#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.
#pragma version = 1.00

#pragma hide=1

// use this to define a demo-mode package
#define DEMO

#ifdef DEMO
Static Constant k_version = -1.00
#else
#include "Radial Profiler Calculator"
Static Constant k_version = 1.00
#endif

// locations for Image Tools Package
Static StrConstant k_fullpackageFolder = "root:Packages:imgT"
Static StrConstant k_imgDisplay = "imgT_display"
Static StrConstant k_fullpanel = "imgT_display#imgT_panel"
Static StrConstant k_panelName = "imgT_panel"

// location for this package
Static StrConstant k_datafolder = "RadialProfiles"
Static StrConstant k_RPPanel = "RP_Inputs"
Static StrConstant k_RPGraphs = "RP_Profiles"
Static StrConstant k_RPFullPanel = "RP_Profiles#RP_Inputs"

// panel and graph sizes
Static Constant pwidth = 280
Static Constant pheight = 560
Static Constant gwidth = 300

//// menu placement
Menu "Packages"
	SubMenu "Image Processing"
		"Radial Profiler", /Q, init_RadialProfiler()
	end
end

// *** STRUCTURES

// radial profiler settings
Structure S_RadialProfilerParams
	variable fedge
	variable mask
	variable cx
	variable cy
	variable rinx
	variable rdouter
	variable circular
	variable aspyx
	variable dx
	variable faveover
	variable method
	variable normalize
	variable overlay
endStructure

// initialize the parameters
Static Function Sf_InitRadialProfilerParams(srp,centeronly)
	STRUCT S_RadialProfilerParams &srp
	variable centeronly
	
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string wdf = "root:" + cwdfolder
	variable left, right, top, bottom, minr, dx

	wave/SDFR=$wdf simg = $cwfile
	left = IndexToScale(simg,0,0)
	right = IndexToScale(simg,inf,0)
	top = IndexToScale(simg,0,1)
	bottom = IndexToScale(simg,inf,1)

	srp.cx = round((right + left)/2)
	srp.cy = round((bottom + top)/2)

	if (centeronly)
		return 0
	endif
	
	dx = DimDelta(simg,0)
	srp.fedge = 0.01
	srp.mask = 1
	minr = min((right - left),(bottom - top))
	srp.rinx = round(minr/(4*dx))
	srp.rdouter = 0.10
	srp.circular = 1
	srp.aspyx = 1
	srp.dx = dx
	
	return 0
end	

// read the parameters from the control panel
// also accessed by Calculator
Function Sf_ReadRadialProfilerParams(srp,[justcursor])
	STRUCT S_RadialProfilerParams &srp
	variable justcursor

	// cursor
	ControlInfo/W=$k_RPPanel setvarcx
	srp.cx = v_value
	ControlInfo/W=$k_RPPanel setvarcy
	srp.cy = v_value
	
	ControlInfo/W=$k_RPPanel setvarmir
	srp.rinx = v_value
	ControlInfo/W=$k_RPPanel setvarmor
	srp.rdouter = v_value
	ControlInfo/W=$k_RPPanel checkCircular
	srp.circular = v_value
	ControlInfo/W=$k_RPPanel valdispaspyx
	srp.aspyx = v_value
	
	if (!ParamIsDefault(justcursor))
		return 0
	endif

	ControlInfo/W=$k_RPPanel setvarEdge
	srp.fedge = v_value
	ControlInfo/W=$k_RPPanel popupRPMask
	srp.mask = v_value

	ControlInfo/W=$k_RPPanel setvaraveo
	srp.faveover = v_value
	ControlInfo/W=$k_RPPanel popupMethod
	srp.method = v_value
	
	ControlInfo/W=$k_RPPanel checknormalize
	srp.normalize = v_value

	ControlInfo/W=$k_RPPanel popupQuadrant
	srp.overlay = v_value
	
	return 0
end

// get rings
// 1 - inner, 2 - outer
// 0 - cx, 1 - cy, 2 - rx, 3 - ry
Static Function/wave Sf_GetRings(ioro)
	variable ioro

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder
	
	string wdf = "root:" + cwdfolder + ":" + k_datafolder
	
	wave/SDFR=$wdf/Z ringi, ringo
	make/N=4/D/FREE ring

	if (!waveexists(ringi))
		ring = NaN
		return ring
	endif

	switch(ioro)
		case 1:	// inner
			ring[0] = ringi[4]
			ring[1] = ringi[5]
			ring[2] = ringi[6]
			ring[3] = ringi[7]
			break
		case 2:	// outer
			ring[0] = ringo[4]
			ring[1] = ringo[5]
			ring[2] = ringo[6]
			ring[3] = ringo[7]
			break
	endswitch
	
	return ring
end

//store rings
Static Function Sf_StoreRings(left,top,right,bottom,cx,cy,rx,ry,ring,[rdouter])
	variable left, top, right, bottom, cx, cy, rx, ry, ring, rdouter

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile
	
	string idf = "root:" + cwdfolder
	string wdf = idf + ":" + k_datafolder
	variable dx
	
	wave/SDFR=$wdf/Z ringi, ringo
	wave/SDFR=$idf simg = $cwfile
	
	if (waveexists(ringi)==0)
		DFREF cdf = GetdataFolderDFR()
		SetDataFolder $idf
		NewDataFolder/O/S $k_datafolder
		make/N=8 ringi, ringo
		SetDataFolder cdf
	endif
	wave/SDFR=$wdf ringi, ringo
	redimension/N=8 ringi, ringo

	switch(ring)
		case -1:		// update inner
			ringi[6] = rx
			ringi[7] = ry
			ringi[0] = ringi[4] - rx
			ringi[1] = ringi[5] - ry
			ringi[2] = ringi[4] + rx
			ringi[3] = ringi[5] + ry
			break
		case 0:		// initialize
			ringi[0] =  cx - rx
			ringi[1] = cy - ry
			ringi[2] = cx + rx
			ringi[3] = cy + ry	
			ringi[4] = cx
			ringi[5] = cy
			ringi[6] = rx
			ringi[7] = ry
			ringo[0] = cx - rx*(1 + rdouter)
			ringo[1] = cy - ry*(1 + rdouter)
			ringo[2] = cx + rx*(1 + rdouter)
			ringo[3] = cy + ry	*(1 + rdouter)
			ringo[4] = cx
			ringo[5] = cy
			ringo[6] = rx*(1 + rdouter)
			ringo[7] = ry*(1 + rdouter)
			break
		case 1:		// inner
			ringi[0] = left
			ringi[1] = top
			ringi[2] = right
			ringi[3] = bottom	
			ringi[4] = cx
			ringi[5] = cy
			ringi[6] = rx
			ringi[7] = ry
			break
		case 2:		// outer
			ringo[0] = left
			ringo[1] = top
			ringo[2] = right
			ringo[3] = bottom	
			ringo[4] = cx
			ringo[5] = cy
			ringo[6] = rx
			ringo[7] = ry
			break
	endswitch
	return 0
end

// *** LISTS AND CHECKS

// threshold is valid
Function f_RPThresholdisValid()

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string wdf = "root:" + cwdfolder
	string thstr

	variable rtn = 0, cplane, thmax
	
	if (f_ThresholdExists())
		if (f_IsThreshold())
			thstr = cwfile
		else
			thstr = cwfile + "_th"
		endif
		wave/SDFR=$wdf thimg = $thstr
		if (f_IsStack())
			cplane = f_PlaneNumber()
			MatrixOP/FREE thfimg = layer(thimg,cplane)
		else
			duplicate/FREE thimg thfimg
		endif
		thmax = wavemax(thfimg)
		if (thmax != 0)
			rtn = 1
		endif
	endif
	
	return rtn
end

// list of profile mask methods
Function/S f_RPMasks()

	string rtstr = "None;"
	
	STRUCT S_MarqueeCoordinates mq
	Sf_GetMQCoordinates(mq)

	if (mq.hasmarquee == 1)
		rtstr += "x<[ ]>x; [>x<] ;"
	endif
	
	return rtstr
end

// does a blob exist?
Function f_RPBlobExists()

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder
	
	string wdf = "root:" + cwdfolder + ":" + k_datafolder
	variable rtn = 0
	
	wave/SDFR=$wdf/Z blob = M_Particle
	if (WaveExists(blob))
		rtn = 1
	endif
	return rtn
end

// returns list of menu options for results
Function/S f_RPLoResults()
	
	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder

	string rstr = "None;"
	
	string wdf = "root:RadialProfilerResults"
	
	if (DataFolderExists(wdf))
		rstr += "Distribution;Blob;All;"
	endif
	
	return rstr
end

// returns list of possible optimize methods
Function/S f_RPLoOptimizeMethods()

	string rstr = "Manually;"

#ifndef DEMO
	rstr += "CoM Image;"
	if (f_RPThresholdisValid())
		rstr += "CoM Threshold;"
	else
		rstr += "\\M1(CoM Threshold;"
	endif
	if (f_RPBlobExists())
		rstr += "CoM Filled;Smallest Circle;"
	else
		rstr += "\\M1(CoM Filled;\\M1(Smallest Circle;"
	endif
#else
	rstr += "\\M1(CoM Image;\\M1(CoM Threshold;\\M1(CoM Filled;\\M1(Smallest Circle;"
#endif
	
	return rstr
end

// are the cursors blocked?
Static Function f_RPCursorsBlocked()

	GetWindow $k_fullPanel needUpdate
	return v_value
end

// *** INITIALIZATIONS

// initialize the package
Function init_RadialProfiler()

	string imgID

	// just show the existing panel
	DoWindow/F $k_RPGraphs
	if (v_flag == 1)
		return 0
	endif
	
	// cannot create without Image Tools
	DoWindow $k_imgDisplay
	if (v_flag == 0)
		DoAlert/T="No Image Tools Window", 0, "You must have an Image Tools Window active to run this package."
		return 0
	endif
	
	// check dimensions
	if (init_RPConfirmImage() < 0)
		DoAlert/T="Incorrect XY Dimension Sizes", 0, "This profiler only works on images with the same (x,y) scaling."
		return 0
	endif
	
	// set up the data folder
	init_RPDataFolder()
	
	// inititalize the rings
	STRUCT S_RadialProfilerParams srp
	init_RPRings(srp)
	
	// create and show panel
	init_RPPanel(srp)

	// update the plot
	RP_MakeRowColProfiles(srp.faveover)
	RP_UpdateRowColPlot()
	
	// set the panel hook	
	imgID = f_DisplayImageID(0)
	SetWindow $k_RPFullPanel hook(RadialProfiler)=Hf_RPPanel, userData(activeimage) = imgID

	return 0
end

// check dimension sizes
Static Function init_RPConfirmImage()

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string wdf = "root:" + cwdfolder
	
	// cannot work on images with mis-matched dimension sizes
	wave/SDFR=$wdf simg = $cwfile
	if (DimDelta(simg,0) != DimDelta(simg,1))
		return -1
	endif
	
	return 0
end

// initialize the datafolder
Static Function init_RPDataFolder()

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string wdf, ddf
	variable cplane = f_PlaneNumber()
	variable newfldr = 0, dx, xo

	wdf = "root:" + cwdfolder
	ddf = wdf + ":" + k_datafolder
	wave/SDFR=$wdf simg = $cwfile

	DFREF cdf = GetDataFolderDFR()
	
	// create new when needed
	if (!DataFolderExists(ddf))
		SetDataFolder $wdf
		NewDataFolder/S $k_datafolder
		newfldr = 1
	else
		SetDataFolder $ddf
	endif
	
	if (f_IsStack())
		MatrixOP/FREE sfimg = fp64(layer(simg,cplane))
	else
		MatrixOP/FREE sfimg = fp64(simg)
	endif
	CopyScales/P simg, sfimg

	// create row / col waves
	dx = DimDelta(simg,0)
	MatrixOP/O sRows = sumRows(sfimg)
	xo = IndextoScale(simg,0,0)
	SetScale/P x, xo, dx, sRows
	MatrixOP/O sCols = sumCols(sfimg)^t
	xo = IndextoScale(simg,0,1)
	SetScale/P x, xo, dx, sCols

	RP_ValidateRings(sfimg)
	
	SetDataFolder cdf
	return newfldr
end

// validate the rings for a new data folder operation
Static Function RP_ValidateRings(simg)
	wave simg
	
	variable npts, gleft, gright, gtop, gbottom
	variable dx, left, right, top, bottom
	
	npts = DimSize(simg,0)
	gleft = IndextoScale(simg,0,0)
	gright = IndextoScale(simg,npts-1,0)
	npts = DimSize(simg,1)
	gtop = IndextoScale(simg,0,1)
	gbottom = IndextoScale(simg,npts-1,1)

	STRUCT S_RadialProfilerParams srp
	Sf_ReadRadialProfilerParams(srp)
	
	left = srp.cx - srp.rinx
	right = srp.cx + srp.rinx
	top = srp.cy - srp.rinx
	bottom = srp.cy + srp.rinx

	if ((left < gleft) || (right > gright) || (top < gtop) || (bottom > gbottom))
		init_RPRings(srp)
		RP_ResetPanelControls(srp)
	endif
	return 0
end

// create the first rings
Static Function init_RPRings(srp,[centeronly])
	STRUCT S_RadialProfilerParams &srp
	variable centeronly
	
	if (ParamIsDefault(centeronly))
		Sf_InitRadialProfilerParams(srp,0)
	else
		Sf_InitRadialProfilerParams(srp,1)
	endif
	Sf_StoreRings(0,0,0,0,srp.cx,srp.cy,srp.rinx,srp.rinx,0,rdouter=srp.rdouter)
	RP_MoveInner(srp.cx, srp.cy, srp.rinx, srp.aspyx, srp.circular)
	RP_SetOuter(srp.rdouter,srp.circular)

	return 0
end

// block the cursors from being updated
Static Function RP_BlockCursorUpdate(how)
	variable how
	
	SetWindow $k_fullPanel needUpdate=how	
	return 0
end

// create and show the panel
Static Function init_RPPanel(srp)
	STRUCT S_RadialProfilerParams &srp

	string vstr = "v " + num2str(k_version)	
	variable voffset = 20, hoffset = 0
	variable thmissing
	
	Display/W=(100,50,gwidth+100,pheight+50)/N=$k_RPGraphs
	NewPanel/EXT=0/HOST=#/W=(0,0,pwidth,pheight)/N=$k_RPPanel as "Radial Profiler"
	ModifyPanel fixedSize=1, noEdit=1
	
	DefaultGuiFont/W=$k_RPPanel all={"_IgorMedium",14,0}, button={"_IgorMedium",16,0}, panel={"_IgorMedium",12,0}
	
	SetDrawEnv fsize=10
	DrawText pwidth-40, voffset-2, vstr
	
	// RING SETUP
	
	SetDrawEnv fsize=16,fstyle=1
	DrawText 10, voffset+5, "Rings"

	thmissing = !f_RPThresholdisValid()
	Button buttonautosetrings,pos={pwidth-60,voffset},size={50,25},proc=RP_ButtonProc,title="Find", disable=(2*thmissing)
	
	voffset+=20
	
	SetDrawEnv fsize=14,fStyle=2
	DrawText 15, voffset+5, "Auto-Locate on Blob Edges"
	
	voffset+=10
	
	SetVariable setvarEdge,pos={20,voffset},size={100,20},proc=RP_SetVarProc,title="Edge", disable=(2*thmissing)
	SetVariable setvarEdge,format="%3.2f",limits={0.01,1,0.01},value= _NUM:0.01

	PopupMenu popupRPMask,pos={140,voffset},size={100,25},title="Mask",disable=(2*thmissing)
	PopupMenu popupRPMask,mode=1,popvalue="None",value= #"f_RPMasks()"
	
	voffset+=30

	Button buttoncenterrings,pos={pwidth-125,voffset},size={60,25},proc=RP_ButtonProc,title="Center"
	Button buttonclearrings,pos={pwidth-60,voffset},size={50,25},proc=RP_ButtonProc,title="Clear"
	
	voffset+=15
		
	SetDrawEnv fsize=14,fStyle=2
	DrawText 15, voffset+5, "Position Manually"
	
	voffset+=15
	
	SetVariable setvarcx,pos={20,voffset},size={110,20},proc=RP_SetVarProc,title="<-c->"
	SetVariable setvarcx,format="%4.0f",limits={-100000,100000,10},value=_NUM:srp.cx
	
	SetVariable setvarcy,pos={150,voffset},size={110,20},proc=RP_SetVarProc,title="∧|c|∨"
	SetVariable setvarcy,format="%4.0f",limits={-100000,100000,10},value=_NUM:srp.cy
	
	voffset+=25
	
	SetVariable setvarmir,pos={20,voffset},size={135,20},proc=RP_SetVarProc,title="\\[0Inner r\\Bx\\M"
	SetVariable setvarmir,format="%4.0f",limits={0,100000,10},value=_NUM:srp.rinx
		
	CheckBox checkCircular pos={pwidth-80,voffset+2},title="Circular", value=srp.circular, proc=RP_CheckProc
	
	voffset+=25

	SetVariable setvarmor,pos={20,voffset},size={135,20},proc=RP_SetVarProc,title="\\[0Outer ∆r\\Bx\\M"
	SetVariable setvarmor,format="%3.2f",limits={-1,2,0.01},value=_NUM:srp.rdouter
	
	//voffset+=10

	ValDisplay valdispAspYX,pos={pwidth-105,voffset},size={90,25},title="||/--"
	ValDisplay valdispAspYX, format="%3.3f",limits={0,0,0},value= _NUM:srp.aspyx
	
	voffset+=30
	
	PopupMenu popupoptimize,pos={15,voffset},size={140,25},title="Optimize",fStyle=2,proc=RP_PopMenuProc
	PopupMenu popupoptimize,mode=1,popvalue="Manually",value=#"f_RPLoOptimizeMethods()"
	
	// PROFILE CALCULATIONS
	
	voffset+=45
	
	SetDrawEnv fsize=16,fstyle=1
	DrawText 10, voffset+5, "Profiles"

	Button buttonanalyze,pos={pwidth-85,voffset},size={75,25},proc=RP_ButtonProc,title="Analyze"

	voffset+=25
	
	SetDrawEnv fsize=14,fStyle=2
	DrawText 15, voffset, "Radial Method"
	
	Checkbox checknormalize, pos={pwidth-95,voffset+5},size={20,20},title="Normalize",value=1, proc=RP_CheckProc, disable=2

	voffset+=5
	
	PopupMenu popupmethod,pos={20,voffset},size={120,25},title="",proc=RP_PopMenuProc
	PopupMenu popupmethod,mode=1,popvalue="Pixels - Linear",value= #"\"Pixels - Linear;Pixels - Quadratic;Line Profile - Circular;\""

	voffset+=45
	
	SetDrawEnv fsize=14,fStyle=2
	DrawText 15, voffset, "Rows/Columns"
	
	SetVariable setvaraveo,pos={pwidth-110,voffset-18},size={95,25},proc=RP_SetVarProc,title="f\Bave\M" //, disable=2
	SetVariable setvaraveo,format="%3.2f",limits={-0.1,0.1,0.01},value=_NUM:0

	voffset+=20

	// RESULTS

	PopupMenu popupshowresults,pos={90,voffset-3},proc=RP_PopMenuProc,title="Show"
	PopupMenu popupshowresults,value=#"f_RPLoResults()",popvalue="None",mode=1

	voffset+=10
	
	SetDrawEnv fsize=16,fstyle=1
	DrawText 10, voffset+5, "Results"
	
	voffset+=20
	
	SetDrawEnv fsize=14,fstyle=2
	DrawText 15, voffset+5, "Distribution"
	
	voffset+=10

	ValDisplay valdisprmp,pos={20,voffset},size={100,25},title="R\Bmp",format="%4.4g", disable=2

	voffset+=25

	ValDisplay valdisprave,pos={20,voffset},size={100,25},title="R\Bave",format="%4.4g", disable=2

	voffset+=25

	ValDisplay valdisprrms,pos={20,voffset},size={100,25},title="R\Brms", format="%4.4g", disable=2

	voffset+=25

	ValDisplay valdisprg,pos={20,voffset},size={100,25},title="R\Bg", format="%4.4g", disable=2

	voffset+=25

	ValDisplay valdispru50,pos={20,voffset},size={100,25},title="R\BU50", format="%4.4g", disable=2

	voffset+=25

	ValDisplay valdisprfwhm,pos={20,voffset},size={100,25},title="R\Bfwhm", format="%4.4g", disable=2

	voffset-=(5*25+10)
	hoffset = 160
	
	SetDrawEnv fsize=14,fstyle=2
	DrawText hoffset, voffset+5, "Blob"
	
	CheckBox checkShowBlob pos={hoffset+40,voffset-13},title="Show",proc=RP_CheckProc, value=0, disable=2
	
	voffset+=10

	ValDisplay valdispUAD,pos={hoffset,voffset},size={100,25},format="%4.4g",title="U\BAD", disable=2

	voffset+=25

	ValDisplay valdispFUniform,pos={hoffset,voffset},size={100,25},format="%3.4f",title="f\BU", disable=2

	voffset+=25

	ValDisplay valdispBlobA,pos={hoffset,voffset},size={90,25},format="%4.0f",title="A", disable=2

	voffset+=25
	ValDisplay valdispBlobP,pos={hoffset,voffset},size={90,25},format="%4.0f",title="P", disable=2

	voffset+=25

	ValDisplay valdispBlobC,pos={hoffset,voffset},size={90,25},format="%3.1f",title="C", disable=2

	voffset+=30

	Button buttonsave, pos={pwidth-90,voffset},size={80,25},proc=RP_ButtonProc,title="Save", disable=2

	voffset+=25
	
	PopupMenu popupquadrant,pos={15,voffset},size={120,25},title="Quadrant", disable=2
	PopupMenu popupquadrant,mode=1,value= #"\"None;Separate;Overlay;\"",proc=RP_PopMenuProc

	Button buttonpublish,pos={pwidth-90,voffset},size={80,25},proc=RP_ButtonProc,title="Publish", disable=2

#ifndef DEMO
	Button buttonanalyze, disable=0
	PopupMenu popupmethod, disable=0
	PopupMenu popupshowresults, disable=0
#else
	Button buttonanalyze, disable=2
	PopupMenu popupmethod, disable=2
	PopupMenu popupshowresults, disable=2
#endif

	RP_ClearResults()
	
	return 0
end

// *** HOOKS

Function Hf_RPCursors(hs)
	STRUCT WMWinHookStruct &hs

	variable hookResult = 0
	STRUCT S_RadialProfilerParams srp
	
	variable cx, cy, rinx
	
	switch(hs.eventCode)
		case 7:	// cursor moved
			if (f_RPCursorsBlocked())
				hookResult = 1
				break
			endif
			cx = xcsr(A,k_imgDisplay)
			cy = vcsr(A,k_imgDisplay)
			SetVariable setvarcx, win=$k_RPPanel, value=_NUM:cx
			SetVariable setvarcy, win=$k_RPPanel, value=_NUM:cy
			Sf_ReadRadialProfilerParams(srp,justcursor=1)
			RP_MoveInner(cx, cy, srp.rinx,srp.aspyx,srp.circular)
			RP_SetOuter(srp.rdouter,srp.circular)
			RP_MakeRowColProfiles(srp.faveover)
			RP_UpdateRowColPlot()
			SetWindow $k_RPFullPanel, userData(rpinfo) = "cursor moved"
			hookResult = 1
		break
	endswitch
	
	return hookResult
end

Function Hf_RPPanel(hs)
	STRUCT WMWinHookStruct &hs

	variable hookResult = 0
	string userData, imgID
	variable plnd, plns
	STRUCT S_RadialProfilerParams srp
	
	switch(hs.eventCode)
		case 0:		// activate
			// check whether image has changed
			userData = GetUserData(k_RPFullPanel,"","activeimage")
			plns = str2num(StringFromList(2,userData,":"))
			imgID = f_DisplayImageID(0)
			plnd = str2num(StringFromList(2,imgID,":"))
			if (cmpstr(userData,imgID) != 0)
				// same plane means this must be a jump to new image or image folder
				RP_BlockCursorUpdate(1)
				init_RPDataFolder()
				//init_RPRings(srp)
				//RP_ResetPanelControls(srp)
				Sf_ReadRadialProfilerParams(srp)
				if (numtype(srp.rinx) == 2)
					// no rings so no further changes to make
					hookResult = 1
					break
				endif
				Sf_StoreRings(0,0,0,0,srp.cx,srp.cy,srp.rinx,srp.rinx,0,rdouter=srp.rdouter)
				RP_MakeRowColProfiles(srp.faveover)
				// reset all settings
				RP_ClearGraphs(1)
				RP_KillBlob()
				RP_UpdateRowColPlot()
				RP_ClearPanel()
				RP_ResetPanelControls(srp)
				PopupMenu popupquadrant, win=$k_RPPanel, mode=1, disable=2
				Button buttonpublish,win=$k_RPPanel, disable=2
				Button buttonSave,win=$k_RPPanel, disable=2
				PopupMenu popupoptimize, win=$k_RPPanel, mode=1
#ifndef DEMO
				RP_MakeRadialQuadrant(1)
#endif
				SetWindow $k_RPFullPanel, userData(activeimage) = imgID
				RP_BlockCursorUpdate(0)
			endif
			// update display (in case cursor position has changed)
			userData = GetUserData(k_RPFullPanel,"","rpinfo")
			if (cmpstr(userData,"cursor moved") == 0)
				RP_ClearGraphs(1)
				Sf_ReadRadialProfilerParams(srp)
				RP_MakeRowColProfiles(srp.faveover)
				RP_UpdateRowColPlot()
				SetWindow $k_RPFullPanel, userData(rpinfo) = ""
			endif
			if (!f_hasMQ())
				PopupMenu popupRPMask, win=$k_RPPanel, mode=1
			endif
			hookResult = 1
			break
		case 2:		// kill
		case 14:		// subwindow kill
			DoWindow/F $k_imgdisplay
			if (v_flag)
				SetWindow $k_imgDisplay hook(rpcursors)=$""
				RP_DrawOval(0,0,0,0,"innerring",kill=1)
				RP_DrawOval(0,0,0,0,"outerring",kill=1)
				RP_PutCenter(0,0,0)
				RemoveImage/W=$k_imgdisplay/Z RP_quadrant
				DoWindow/K/Z RP_Blob
			endif
			hookResult = 1
			break
	endswitch

	return hookResult
end

// *** GENERAL PURPOSE

Static Function RP_KillBlob()

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder
	
	string wdf = "root:" + cwdfolder + ":" + k_datafolder
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder $wdf
	killwaves/Z M_Particle
	SetDataFolder cdf
	
	return 0
end

// clear the result values in the panel
Static Function RP_ClearResults()

	ValDisplay valdisprmp, win=$k_RPPanel, value=_NUM:NaN, disable=2
	ValDisplay valdisprave, win=$k_RPPanel, value=_NUM:NaN, disable=2
	ValDisplay valdisprrms, win=$k_RPPanel, value=_NUM:NaN, disable=2
	ValDisplay valdisprg, win=$k_RPPanel, value=_NUM:NaN, disable=2
	ValDisplay valdisprU50, win=$k_RPPanel, value=_NUM:NaN, disable=2
	ValDisplay valdisprfwhm, win=$k_RPPanel, value=_NUM:NaN, disable=2
	ValDisplay valdispUAD, win=$k_RPPanel, value=_NUM:NaN, disable=2
	ValDisplay valdispFUniform, win=$k_RPPanel, value=_NUM:NaN, disable=2
	ValDisplay valdispBlobA, win=$k_RPPanel, value=_NUM:NaN, disable=2
	ValDisplay valdispBlobP, win=$k_RPPanel, value=_NUM:NaN, disable=2
	ValDisplay valdispBlobC, win=$k_RPPanel, value=_NUM:NaN, disable=2
	
	return 0
end

// *** TARGET AND RINGS

// put/remove the center cursor
Static Function RP_PutCenter(cx, cy, show)
	variable cx, cy, show
	
	string cImage = ""
	switch(show)
		case 0:		// kill
			DoWindow $k_imgDisplay
			if (v_flag)
				SetWindow $k_imgDisplay hook(rpcursors)=$""
				Cursor/W=$k_imgDisplay/K A
			endif
			break
		case 1:
			cImage = f_CurrentImage()
			Cursor/W=$k_imgDisplay/I/H=1/S=0/T=3 A $cImage cx, cy
			SetWindow $k_imgDisplay hook(rpcursors)=Hf_RPCursors
			break
	endswitch
	return 0
end

// draw an oval
Static Function RP_DrawOval(left,top,right,bottom,oname,[kill])
	variable left, top, right, bottom, kill
	string oname
	
	if (!ParamIsDefault(kill))
		DoWindow $k_imgDisplay
		if (v_flag)
			SetDrawLayer/W=$k_imgDisplay ProgFront
			DrawAction/W=$k_imgDisplay getgroup=$oname, delete
		endif
		return 0
	endif
		
	SetDrawLayer/W=$k_imgDisplay ProgFront
	DrawAction/W=$k_imgDisplay getgroup=$oname, delete, begininsert
	SetDrawEnv/W=$k_imgDisplay gstart, gname=$oname
	SetDrawEnv/W=$k_imgDisplay xcoord=top, ycoord=left
	SetDrawEnv/W=$k_imgDisplay linefgc=(65535,65535,0),fillpat=0,linethick=3
	DrawOval/W=$k_imgDisplay left,top,right,bottom
	SetDrawEnv/W=$k_imgDisplay gstop
	SetDrawLayer/W=$k_imgDisplay UserFront
	return 0
end

// set the target ring
Static Function RP_SetTarget(srp)
	STRUCT S_RadialProfilerParams &srp

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	STRUCT S_MarqueeCoordinates smq
		
	string wdf, swave, thwave
	variable axo, axd, ayo, ayd
	variable cplane, nump, zeroxo, zeroxf, zeroyo, zeroyf
	variable widthx, widthy, rx, ry, wmax
	variable left, top, right, bottom, osize = 0
	
	if (f_RPThresholdIsValid())
		if (f_isThreshold())
			thwave = cwfile
			swave = ReplaceString("_th",cwfile,"")
		else
			swave = cwfile
			thwave = swave + "_th"
		endif
	else
		DoAlert/T="Image Processing", 0, "The threshold for this image is invalid. Set a new one first."
		return 0
	endif
	
	wdf = "root:" + cwdfolder
	wave/SDFR=$wdf srimg = $swave
	wave/SDFR=$wdf timg = $thwave
	if (f_IsStack())
		cplane = f_PlaneNumber()
		MatrixOP/FREE simg = fp64(srimg[][][cplane])
		MatrixOP/FREE thimg = fp64(timg[][][cplane])
		duplicate/FREE thimg thximg
	else
		duplicate/FREE srimg, simg
		duplicate/FREE timg, thimg, thximg
		redimension/D simg, thimg, thximg
	endif
	CopyScales/P srimg, simg, thimg, thximg
	
	switch(srp.mask)
		case 2:		// mask outside
			Sf_GetMQCoordinates(smq)
			left = ScaletoIndex(thimg,smq.pgleft,0)
			right = ScaletoIndex(thimg,smq.pgright,0)
			top = ScaletoIndex(thimg,smq.pgtop,1)
			bottom = ScaletoIndex(thimg,smq.pgbottom,1)		
			thimg = 0
			thimg[left,right][top,bottom] = thximg
			break
		case 3:		// mask inside
			Sf_GetMQCoordinates(smq)
			left = ScaletoIndex(thimg,smq.pgleft,0)
			right = ScaletoIndex(thimg,smq.pgright,0)
			top = ScaletoIndex(thimg,smq.pgtop,1)
			bottom = ScaletoIndex(thimg,smq.pgbottom,1)		
			thimg[left,right][top,bottom] = 0
			break
	endswitch
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder $wdf
	NewDataFolder/O/S $k_datafolder
	
	wave sRows, sCols
	
	// do rows
	// set scale for rows
	axo = DimOffset(simg,0)
	axd = DimDelta(simg,0)
	SetScale/P x, axo,axd, sRows
	
	MatrixOP/FREE tRows = sumRows(thimg)
	SetScale/P x, axo,axd, tRows	

	MatrixOP/FREE mRows = maxRows(simg)
	
	nump = numpnts(tRows)
	WaveStats/Q/M=1 tRows
	FindLevel/EDGE=1/Q tRows, srp.fedge*V_max
	zeroxo = round(v_LevelX)
	FindLevel/EDGE=2/R=[nump-1,0]/Q tRows, srp.fedge*V_max
	zeroxf = round(v_LevelX)
	rx = (zeroxf - zeroxo)/2
	widthx = 2*rx
	srp.cx = round((zeroxf + zeroxo)/2)
	srp.rinx = rx
		
	// do columns
	// set scale for columns
	ayo = DimOffset(simg,1)
	ayd = DimDelta(simg,1)
	SetScale/P x, ayo,ayd, sCols
	MatrixOP/FREE tCols = sumCols(thimg)^t
	SetScale/P x, axo,axd, tCols
		
	MatrixOP/FREE mCols = maxCols(simg)^t

	nump = numpnts(tCols)
	WaveStats/Q/M=1 tCols
	FindLevel/EDGE=1/Q tCols, srp.fedge*V_max
	zeroyo = round(v_LevelX)
	FindLevel/EDGE=2/R=[nump-1,0]/Q tCols, srp.fedge*V_max
	zeroyf = round(v_LevelX)
	ry = (zeroyf - zeroyo)/2
	widthy = 2*ry
	srp.cy = round((zeroyf + zeroyo)/2)
	srp.aspyx = widthy/widthx
	
	wmax = max(widthx, widthy)
	if (srp.circular)
		widthx = wmax
		widthy = wmax
		rx = ceil(wmax/2)
		ry = rx
	endif
	
	left = round(srp.cx - (widthx/2))
	right = round(left + widthx)
	
	// restrict left
	if (left < axo)
		right += left
		left = axo
		widthx = right - left
		osize = 1
	endif
	//restrict right
	wmax = DimSize(simg,0) - 1
	if (right > wmax)
		left += wmax - right
		right = wmax
		widthx = right - left
		osize = 1
	endif
	top = round(srp.cy - (widthy/2))
	bottom = round(top + widthy)
	// restrict top
	if (top < ayo)
		bottom += top
		top = 0
		widthy = bottom - top
		osize = 1
	endif
	// restrict bobbom
	wmax = DimSize(simg,1) - 1
	if (bottom > wmax)
		top += wmax - bottom
		bottom = wmax
		widthy = bottom - top
		osize = 1
	endif
	
	if (osize && srp.circular)
		wmax = min(widthx,widthy)
		widthx = wmax
		widthy = wmax
		left = round(srp.cx - widthx/2)
		right = round(srp.cx + widthx/2)
		top = round(srp.cy - widthy/2)
		bottom = round(srp.cy + widthy/2)
	endif
	
	// set rings
	RP_PutCenter(srp.cx, srp.cy, 1)
	RP_DrawOval(left, top, right, bottom, "innerring")
	Sf_StoreRings(left,top,right,bottom,srp.cx,srp.cy,rx,ry,1)
	if (numtype(srp.rdouter)==2)
		srp.rdouter = 0.1
	endif
	RP_SetOuter(srp.rdouter,srp.circular)
	
	SetDataFolder cdf
	
	return 0
end

Static Function RP_MoveInner(cx, cy, rinx, aspyx, circular)
	variable cx, cy, rinx, aspyx, circular
	
	variable rx, ry
	variable drx, dry
	variable left, top, right, bottom
	
	rx = Sf_GetRings(1)[2]
	// no rings set yet
	if (numtype(rx) == 2)
		rx = rinx
		ry = rinx
	else
		ry = Sf_GetRings(1)[3]
	endif
	
	if (circular)
		aspyx = 1
		ry = rx
	else
		ry = rx*aspyx
	endif
	
	drx = rinx - rx
	dry = drx*aspyx
	
	rx += drx
	ry += dry
	
	left = cx - rx
	right = cx + rx
	top = cy - ry
	bottom = cy + ry

	RP_PutCenter(cx,cy,1)
	RP_DrawOval(left, top, right, bottom, "innerring")
	Sf_StoreRings(left,top,right,bottom,cx,cy,rx,ry,1)
	
	return 0
end

// set the outer ring
// mpcf - max expansion factor
// circular - circular or not
Static Function RP_SetOuter(mpcf, circular)
	variable mpcf, circular
	
	variable cx, cy, rx, ry
	variable left, top, right, bottom
	
	mpcf += 1
	
	cx = Sf_GetRings(1)[0]
	cy = Sf_GetRings(1)[1]
	rx = floor(mpcf*Sf_GetRings(1)[2])
	ry = floor(mpcf*Sf_GetRings(1)[3])
	
	left = cx - rx
	right = cx + rx
	top = cy - ry
	bottom = cy + ry

	RP_DrawOval(left, top, right, bottom, "outerring")
	Sf_StoreRings(left,top,right,bottom,cx,cy,rx,ry,2)
	
	return 0
end

// clear the rings
Static Function RP_ClearRings()
	
	// clear the graphs
	RP_DrawOval(0,0,0,0,"innerring",kill=1)
	RP_DrawOval(0,0,0,0,"outerring",kill=1)
	RP_PutCenter(0,0,0)
	return 0
end

// *** PROFILES

// create row + colume profiles
// ao - average over
Static Function RP_MakeRowColProfiles(ao)
	variable ao

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder, cwfile

	string wdf = "root:" + cwdfolder + ":" + k_datafolder
	
	variable xleft, xtop, xright, xbottom
	variable xlimitl, xlimitt, xlimitr, xlimitb
	variable wpxmax, wpymax, npo, po
	variable dx, dy, wx, wy, wxo, wxf, wyo, wyf
	variable rave, lave, mbl, bbl
	variable maxR, maxC
	
	// set waves to work
	
	DFREF cdf = GetDataFolderDFR()
	SetDataFolder $wdf
	
	variable cx, cy
	wave ringo, sCols, sRows
	
	// outer ring values set profile limits
	xleft = ringo[0]
	xtop = ringo[1]
	xright = ringo[2]
	xbottom = ringo[3]
	cx = ringo[4]
	cy = ringo[5]
	dx = DimDelta(sRows,0)
	wx = xright - xleft
	dy = DimDelta(sCols,0)
	wy = xbottom - xtop
	
	// number of points to add or subtract to averaging
	wpxmax = numpnts(sRows)
	wpymax = numpnts(sCols)
	if (ao < 0)
		npo = -round(min(-ao*wpxmax,-ao*wpymax))
	else
		npo = round(min(ao*wpxmax,ao*wpymax))
	endif

	// ROWS
	// right side
	wxf = pnt2x(sRows,wpxmax-1)
	xlimitr = xright + npo*dx
	xlimitr = xlimitr > wxf ? wxf : xlimitr
	WaveStats/Q/R=(xright,xlimitr) sRows
	rave = V_avg	
	// left side
	wxo = pnt2x(sRows,0)
	xlimitl = xleft - npo*dx
	xlimitl = xlimitl < wxo ? wxo : xlimitl
	WaveStats/Q/R=(xlimitl,xleft) sRows
	lave = V_avg
	// baseline
	mbl = (rave - lave)/wx
	bbl = lave - mbl*xleft
	Duplicate/O sRows zRows
	zRows = sRows - (mbl*x + bbl)
	
	Duplicate/FREE zRows zsRows
	po = x2pnt(sRows,xleft)
	if (po > 0)
		zsRows[0,po]=0
	endif
	po = x2pnt(sRows,xright)
	if (po < wpxmax - 1)
		zsRows[po,wpxmax-1] = 0
	endif
	Sort/R zsRows, zsRows
	SetScale/P x, 0, 1, zsRows
	maxR = sum(zsRows,0,(abs(npo)))/(abs(npo)+1)
	zRows /= maxR
	RP_SnipProfile(zRows, xleft, xright)
	
	// COLS
	// bottom
	wyf = pnt2x(sCols,wpymax-1)
	xlimitb = xbottom + npo*dy
	xlimitb = xlimitb > wyf ? wyf : xlimitb
	WaveStats/Q/R=(xbottom,xlimitb) sCols
	rave = V_avg
	// top
	wyo = pnt2x(sCols,0)
	xlimitt = xtop - npo*dy
	xlimitt = xlimitt < wyo ? wyo : xlimitt
	WaveStats/Q/R=(xlimitt,xtop) sCols
	lave = V_avg
	// baseline
	mbl = (rave - lave)/wy
	bbl = rave - mbl*xbottom
	Duplicate/O sCols zCols
	zCols = sCols - (mbl*x + bbl)
	
	Duplicate/FREE zCols zsCols
	po = x2pnt(zCols,xtop)
	if (po > 0)
		zsCols[0,po]=0
	endif
	po = x2pnt(zCols,xbottom)
	if (po < wpymax - 1)
		zsCols[po,wpymax-1] = 0
	endif
	Sort/R zsCols, zscols
	SetScale/P x, 0, 1, zsCols
	maxC = sum(zsCols,0,abs(npo))/(abs(npo)+1)
	zCols /= maxC
	RP_SnipProfile(zCols, xtop, xbottom)
	
	SetDataFolder cdf
	
	return 0
end

// update the row/column plot
Static Function RP_UpdateRowColPlot([clear])
	variable clear

	DFREF pf = $k_fullpackageFolder
	SVAR/SDFR=pf cwdfolder
	
	string wdf = "root:" + cwdfolder + ":RadialProfiles"
	variable crsL, crsT, crsR, crsB
	
	// cannot update when data folder does not exist
	if (!DataFolderExists(wdf))
		return 0
	endif
	
	wave/SDFR=$wdf/Z zCols, zRows, ringi
	
	if (!waveexists(zCols))
		return 0
	endif
	
	crsL = ringi[0]
	crsT = ringi[1]
	crsR = ringi[2]
	crsB = ringi[3]
	
	duplicate/FREE zCols zmC
	duplicate/FREE zRows zmR
	
	if (!ParamIsDefault(clear))
		ModifyGraph/W=$k_RPGraphs/Z hideTrace(zCols)=1, hideTrace(zRows)=1
		return 0
	endif
	
	if (ItemsInList(TraceNameList(k_RPGraphs,";",1))!=0)
		ReplaceWave/W=$k_RPGraphs trace=zCols, zCols
		ReplaceWave/W=$k_RPGraphs trace=zRows, zRows
		ModifyGraph/Z/W=$k_RPGraphs hideTrace(zCols)=0, hideTrace(zRows)=0
		Tag/W=$k_RPGraphs/C/N=LEFT/F=0/A=RB/L=1/TL={dash=3}/X=0/Y=20 zRows, crsL,"L"
		Tag/W=$k_RPGraphs/C/N=RIGHT/F=0/A=LB/L=1/TL={dash=3}/X=0/Y=20 zRows, crsR,"R"
		Tag/W=$k_RPGraphs/C/N=TOP/F=0/A=RB/L=1/TL={dash=3}/X=0/Y=20 zCols, crsT,"T"
		Tag/W=$k_RPGraphs/C/N=BOTTOM/F=0/A=LB/L=1/TL={dash=3}/X=0/Y=20 zCols, crsB,"B"
	else
		AppendtoGraph/W=$k_RPGraphs zCols,zRows
		ModifyGraph/W=$k_RPGraphs rgb(zCols)=(1,4,52428),rgb(zRows)=(52428,1,1), zero(left)=4
		ModifyGraph/W=$k_RPGraphs mirror(left)=2,mirror(bottom)=2,nticks(left)=8
		ModifyGraph/W=$k_RPGraphs lblPos(left)=73,lblPos(bottom)=47
		ModifyGraph/W=$k_RPGraphs axisEnab(left)={0,0.49},mirror(bottom)=0,axOffset(left)=-2
		SetAxis/W=$k_RPGraphs left -0.2,1.2
		TextBox/W=$k_RPGraphs/C/N=profiles/F=0/A=RC/X=5/Y=-10 "\\K(1,4,52428)blue >|\r\\K(52428,1,1)red   \\[0v\\X0_"
		ModifyGraph/Z/W=$k_RPGraphs hideTrace(zCols)=0, hideTrace(zRows)=0
		Tag/W=$k_RPGraphs/C/N=LEFT/F=0/A=RB/L=1/TL={dash=3}/X=0/Y=20 zRows, crsL,"L"
		Tag/W=$k_RPGraphs/C/N=RIGHT/F=0/A=LB/L=1/TL={dash=3}/X=0/Y=20 zRows, crsR,"R"
		Tag/W=$k_RPGraphs/C/N=TOP/F=0/A=RB/L=1/TL={dash=3}/X=0/Y=20 zCols, crsT,"T"
		Tag/W=$k_RPGraphs/C/N=BOTTOM/F=0/A=LB/L=1/TL={dash=3}/X=0/Y=20 zCols, crsB,"B"
	endif
	
	return 0
end

// snip row/col profiles
// input values are x values of ww
Static Function RP_SnipProfile(ww,xskeep,xekeep)
	wave ww
	variable xskeep, xekeep
	
	variable po, pf, pm
	
	po = x2pnt(ww,xskeep) - 1
	pf = x2pnt(ww,xekeep) + 1
	pm = numpnts(ww)-1
	
	// clip left
	if (po > 0)
		ww[0,po] = NaN
	endif
	
	// clip right
	if (pf < pm)	
		ww[pf,pm] = NaN
	endif
	
	return 0
end

// *** PANEL CONTROLS

// button procedure
Function RP_ButtonProc(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	variable cx, cy, rinx, method, normalize
	string CList
	STRUCT S_RadialProfilerParams srp
	
	switch( ba.eventCode )
		case 2: // mouse up
			// click code here
			
			strswitch(ba.ctrlName)
				case "buttonautosetrings":
					// check that marquee has not disappeared in the meantime
					if (!f_hasMQ())
						PopupMenu popupRPMask, win=$k_RPPanel, mode=1
					endif
					// put inner and outer rings
					RP_ClearGraphs(1)
					Sf_ReadRadialProfilerParams(srp)
					RP_SetTarget(srp)
					RP_ResetPanelControls(srp)
					// make the profiles
					RP_MakeRowColProfiles(srp.faveover)
					break
				case "buttonclearrings":
					RP_ClearGraphs(1)
					RP_ClearPanel()
					RP_ClearRings()
#ifndef DEMO
					RP_MakeRadialQuadrant(1)
#endif
					break
				case "buttoncenterrings":
					ControlInfo/W=$k_RPPanel checkcircular
					if (v_disable == 0)
						Sf_ReadRadialProfilerParams(srp)
						init_RPRings(srp,centeronly=1)
					else
						init_RPRings(srp)
					endif
					RP_ResetPanelControls(srp)
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
#ifndef DEMO
					RP_MakeRadialQuadrant(1)
#endif
					break
				case "buttonanalyze":
#ifndef DEMO
					RP_AnalyzeProfile()
					RP_ShowControls()
					RP_MakeRadialQuadrant(1)
#endif
					break
				case "buttonsave":
#ifndef DEMO
					RP_SaveResults()
#else
					DoAlert/T="Demo Version" 0, "This option is unavailable in the demo version."
#endif
					break
				case "buttonpublish":
#ifndef DEMO
					RP_PublishRadialProfile()
#else
					DoAlert/T="Demo Version" 0, "This option is unavailable in the demo version."
#endif
					break
			endswitch
				break
		case -1: // control being killed
			break
	endswitch

	return 0
end

Function RP_SetVarProc(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva

	STRUCT S_RadialProfilerParams srp
	variable rmax
	
	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
		case 3: // Live update
			strswitch(sva.ctrlName)
				case "setvarEdge":		// edge
					// reset the rings
					Sf_ReadRadialProfilerParams(srp)
					RP_SetTarget(srp)
					RP_SetOuter(srp.rdouter,srp.circular)
					// reset the profiles
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
					// reset the panel
					RP_ResetPanelControls(srp)
					Button buttonpublish, win=$k_RPPanel, disable=2
					PopupMenu popupquadrant, win=$k_RPPanel, mode=1,disable=2
#ifndef DEMO
					RP_MakeRadialQuadrant(1)
#endif
					break
				case "setvarcx":
				case "setvarcy":
					RP_BlockCursorUpdate(1)
					Sf_ReadRadialProfilerParams(srp)
					RP_MoveInner(srp.cx, srp.cy, srp.rinx, srp.aspyx, srp.circular)		
					RP_SetOuter(srp.rdouter,srp.circular)				
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
					RP_ClearResults()
					PopupMenu popupquadrant, win=$k_RPPanel, mode=1,disable=2
#ifndef DEMO
					RP_MakeRadialQuadrant(1)
#endif
					RP_BlockCursorUpdate(0)
					break
				case "setvarmir":		// inner ring position
					// reset the rings
					Sf_ReadRadialProfilerParams(srp)
					RP_MoveInner(srp.cx, srp.cy,sva.dval, srp.aspyx, srp.circular)		
					RP_SetOuter(srp.rdouter,srp.circular)
					// reset the profiles
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
					// reset the panel
					RP_ClearResults()	
					//ValDisplay valdispAspYX, win=$k_RPPanel, value=_NUM:aspyx, disable=0
					Button buttonpublish, win=$k_RPPanel, disable=2
					PopupMenu popupquadrant, win=$k_RPPanel, mode=1,disable=2
#ifndef DEMO
					RP_PlotNormalizedProfile(clear=1)
					RP_MakeRadialQuadrant(1)
#endif
					break
				case "setvarmor":		// outer ring position
					// reset the rings
					Sf_ReadRadialProfilerParams(srp)
					RP_SetOuter(srp.rdouter,srp.circular)
					// reset the profiles
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
					// reset the panel
					RP_ClearResults()
					Button buttonpublish, win=$k_RPPanel, disable=2
					PopupMenu popupquadrant, win=$k_RPPanel, mode=1,disable=2
#ifndef DEMO
					RP_PlotNormalizedProfile(clear=1)
					RP_MakeRadialQuadrant(1)
#endif
					break
				case "setvaraveo":		// average over for baseline
					// reset the profiles
					RP_MakeRowColProfiles(sva.dval)
					RP_UpdateRowColPlot()
					break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch

	return 0
end

// check procedure
Function RP_CheckProc(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba

	STRUCT S_RadialProfilerParams srp
	variable rmax
	
	switch( cba.eventCode )
		case 2: // mouse up
			strswitch(cba.ctrlName)
				case "checkNormalize":
#ifndef DEMO
					RP_AnalyzeProfile(renormalize=1)
#endif
					break
				case "checkCircular":
					Sf_ReadRadialProfilerParams(srp)
					RP_MoveInner(srp.cx, srp.cy, srp.rinx,srp.aspyx,srp.circular)
					RP_SetOuter(srp.rdouter,srp.circular)
					RP_MakeRowColProfiles(srp.faveover)
					RP_UpdateRowColPlot()
					break
				case "checkShowBlob":
#ifndef DEMO	
					RP_ShowBlob(cba.checked)
#endif
					break
				endswitch
		case -1: // control being killed
			break
	endswitch

	return 0
end

Function RP_PopMenuProc(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	STRUCT S_RadialProfilerParams srp

	switch( pa.eventCode )
		case 2: // mouse up
			strswitch(pa.ctrlname)
				case "popupQuadrant":
#ifndef DEMO
					RP_MakeRadialQuadrant(pa.popNum)
#else
					DoAlert/T="Demo Version" 0, "This option is unavailable in the demo version."
#endif
					break
				case "popupMethod":
#ifndef DEMO
					RP_AnalyzeProfile()
					RP_ShowControls()
#endif
					break
				case "popupoptimize":
#ifndef DEMO
					if (pa.popNum == 1)
						SetVariable setvarcx, win=$k_RPPanel, disable=0
						SetVariable setvarcy, win=$k_RPPanel, disable=0
						SetVariable setvarmir, win=$k_RPPanel, disable=0
					else
						// optimize the profile position
						RP_OptimizeTargetCircle(pa.popNum - 1)
						// reset the panel with the new values
						SetVariable setvarcx, win=$k_RPPanel, value=_NUM:Sf_GetRings(1)[0], disable=2
						SetVariable setvarcy, win=$k_RPPanel, value=_NUM:Sf_GetRings(1)[1], disable=2
						SetVariable setvarmir, win=$k_RPPanel, value=_NUM:Sf_GetRings(1)[2], disable=2
						// set the rings to the new parameters
						Sf_ReadRadialProfilerParams(srp)
						RP_MoveInner(srp.cx, srp.cy, srp.rinx, srp.aspyx, srp.circular)
						RP_SetOuter(srp.rdouter,srp.circular)				
						// update the graphs
						RP_MakeRowColProfiles(srp.faveover)
						RP_UpdateRowColPlot()
						RP_ClearResults()
						RP_PlotNormalizedProfile(clear=1)
						PopupMenu popupquadrant,win=$k_RPPanel, disable=2, mode=1
						Button buttonpublish,win=$k_RPPanel, disable=2
						Button buttonSave,win=$k_RPPanel, disable=2
						CheckBox checkShowBlob, win=$k_RPPanel, value=0, disable=2
					endif
#else
					DoAlert/T="Demo Version" 0, "This option is unavailable in the demo version."
#endif
					break
				case "popupShowResults":
#ifndef DEMO
					if (RP_ShowResults(pa.popNum - 1) < 0)
						PopupMenu popupShowResults,win=$k_RPPanel, popvalue="None", mode=1
					endif
#else
					PopupMenu popupShowResults,win=$k_RPPanel, popvalue="None", mode=1, disable=2
#endif
					break
			endswitch
			break
		case -1: // control being killed
			break
	endswitch

	return 0
end

// show the controls
Static Function RP_ShowControls()

	string CList
	CList = ControlNameList(k_RPPanel,";","valdisp*")
	ModifyControlList CList, win=$k_RPPanel, disable=0
	Button buttonpublish,win=$k_RPPanel, disable=0
	Button buttonSave,win=$k_RPPanel, disable=0
	PopupMenu popupquadrant, win=$k_RPPanel, disable=0			
	PopupMenu popupoptimize,win=$k_RPPanel, disable=0
	CheckBox checknormalize, win=$k_RPPanel, disable=0
	if (f_RPBlobExists())
		CheckBox checkShowBlob, win=$k_RPPanel, disable=0
	else
		CheckBox checkShowBlob, win=$k_RPPanel, disable=2
	endif
	if (f_RPThresholdIsValid())
		Button buttonautosetrings,win=$k_RPPanel, disable=0
		PopupMenu popupRPMask, win=$k_RPPanel, disable=0
		SetVariable setvaredge, win=$k_RPPanel, disable=0
	else
		Button buttonautosetrings,win=$k_RPPanel, disable=2
		PopupMenu popupRPMask, win=$k_RPPanel, disable=2
		SetVariable setvaredge, win=$k_RPPanel, disable=2
	endif
	
	return 0
end

// clear the panel (called from button)
Static Function RP_ClearPanel()

	string CList
	
	// clear the control panel
	CList = ControlNameList(k_RPPanel,";","setvar*")
	ModifyControlList CList, win=$k_RPPanel, disable=2
	CList = ControlNameList(k_RPPanel,";","valdisp*")
	ModifyControlList CList, win=$k_RPPanel, disable=2
	CheckBox checkcircular, win=$k_RPPanel, disable=2
	CheckBox checknormalize, win=$k_RPPanel, disable=2
	Button buttonanalyze,win=$k_RPPanel, disable=2
	Button buttonpublish,win=$k_RPPanel, disable=2
	Button buttonSave,win=$k_RPPanel, disable=2
	if (f_RPThresholdIsValid())
		Button buttonautosetrings,win=$k_RPPanel, disable=0
		PopupMenu popupRPMask, win=$k_RPPanel, disable=0
		SetVariable setvaredge, win=$k_RPPanel, disable=0
	else
		Button buttonautosetrings,win=$k_RPPanel, disable=2
		PopupMenu popupRPMask, win=$k_RPPanel, disable=2
		SetVariable setvaredge, win=$k_RPPanel, disable=2
	endif
	PopupMenu popupRPMask,win=$k_RPPanel, disable=2, mode=1
	PopupMenu popupquadrant,win=$k_RPPanel, disable=2, mode=1
	PopupMenu popupmethod, win=$k_RPPanel, disable=2
	PopupMenu popupoptimize,win=$k_RPPanel, mode=1, disable=2
	SetVariable setvarcx, win=$k_RPPanel, value=_NUM:NaN
	SetVariable setvarcy, win=$k_RPPanel, value=_NUM:NaN
	SetVariable setvarmir, win=$k_RPPanel, value=_NUM:NaN
	SetVariable setvarmor, win=$k_RPPanel, value=_NUM:NaN
	RP_ClearResults()
	return 0
end

// reset the panel controls
Static Function RP_ResetPanelControls(srp)
	STRUCT S_RadialProfilerParams &srp
	
	string CList
	variable thmissing = !f_RPThresholdIsValid()
		
	// update the control panel
	CList = ControlNameList(k_RPPanel,";","setvar*")
	ModifyControlList CList, win=$k_RPPanel, disable=0
	ValDisplay valdispAspYX, win=$k_RPPanel, disable=0
	CheckBox checkcircular, win=$k_RPPanel, disable=0
	PopupMenu popupoptimize, win=$k_RPPanel, disable=0
	
	PopupMenu popupRPMask,win=$k_RPPanel, disable=(2*thmissing)
	SetVariable setvarEdge, win=$k_RPPanel, value=_NUM:srp.fedge, disable=(2*thmissing)
	
	SetVariable setvarcx, win=$k_RPPanel, value=_NUM:srp.cx
	SetVariable setvarcy, win=$k_RPPanel, value=_NUM:srp.cy
	SetVariable setvarmir, win=$k_RPPanel, value=_NUM:srp.rinx
	SetVariable setvarmor, win=$k_RPPanel, value=_NUM:srp.rdouter
	SetVariable setvaraveo, win=$k_RPPanel, value=_NUM:srp.faveover
	ValDisplay valdispaspyx, win=$k_RPPanel, value=_NUM:srp.aspyx
#ifndef DEMO
	PopupMenu popupmethod, win=$k_RPPanel, disable=0
	Button buttonanalyze,win=$k_RPPanel, disable=0
	PopupMenu popupquadrant, win=$k_RPPanel, mode=1,disable=0
#endif
	return 0
end

// clear (all or only limited)
Static Function RP_ClearGraphs(all)
	variable all

	if (all == 1)
		RP_UpdateRowColPlot(clear=1)
	endif
#ifndef DEMO
	RP_PlotNormalizedProfile(clear=1)
#endif
	return 0
end
